#ifndef ATOM_H_
#define ATOM_H_

#include "component.h"
#include <vector>
#include <assert.h>
#include <pthread.h>
#include <stdio.h>
#include <bip-time.h>
class CtrlState;
class Port;
class Connector;
class Priority;
class Scheduler;
class Clock;
class LogicalClock;
class TimeConstraint;
class Compound;
class Thread;
/***********************************
 * Class: Atom
 * Description: Atomic component 
 ***********************************/
class Atom : public Component {

  /*** Public data members ***/
  public:
    enum Status {BUSY = 0, READY = 1};

    typedef std::vector<Connector*> connectors_type;
    typedef std::vector<Priority*> priorities_type;
    typedef std::vector<Clock*>  clocks_type;
    typedef std::vector<TimeConstraint*>  constraints_type;	
	typedef std::vector<Port*>  ports_type;

    connectors_type mLinks;      /* Atom links (connectors) */
    priorities_type mConditions;  /* atom conditions */
    clocks_type mClocks;
    constraints_type mConstraints;
    ports_type     mPorts;       /* Atom ports */
    ports_type mNextPorts;
    Port*      mElectedInternalPort;
    ports_type  mNotifiedPorts;
    CtrlState*             BIP_STATE;   /* current control-state */
    Scheduler*             mScheduler;  /* back pointer to scheduler */
    Status                 mStatus;     /* status wrt schduler */
    bool                   mSelected;   /* selected for execution */
    bool                   mExecuteInternal;
   
  protected:
    Thread*                mThreadHolder;
   
  /*** Constructoris & Dectructors ***/
  public:
    Atom(const char*, Compound*,  const LogicalClock*);
    virtual ~Atom();
 
  /*** Public Data-Access Methods ***/
  public:
    connectors_type& links()
    {
      return mLinks;
    } 
    ports_type& nextPorts()
       {
         return mNextPorts;
       } 
    ports_type& ports()
    {
    	return mPorts;
    }
    ports_type& notifiedPorts()
    {
    	return mNotifiedPorts;
    }
    Port* electedInternalPort() {
        return mElectedInternalPort;
    }
    void setElectedInternalPort(Port* port);
    inline Status status() const { return mStatus; }
    inline void setStatus(const Status status) { mStatus = status; }
  
    inline Thread* threadHolder()  { return mThreadHolder; }
    void   setThreadHolder(Thread* thread) { mThreadHolder = thread; }

    inline bool selected() const    { return mSelected; }
    inline void setSelected(bool b) { mSelected = b; }
    
    inline void setExecuteInternal(bool b) { mExecuteInternal = b; }
   
  /*** Public Utility Methods ***/
  public:
    virtual bool       isAtom() const     { return true; }
    virtual bool       isCompound() const { return false; }
            void       resetLink();      /* mark links for re-evaluation */
            void       addPort(Port *p); /*      */
    

  /*** Public Methods visible from appln. ***/
  public:
    virtual void getBusyAtoms(Scheduler*);
        
   
    virtual void setup(Scheduler*);
    virtual void createDefaultThreads();

    virtual bool isExecutingInternal() ;
    virtual void goBusy();
    virtual void goReady();

    virtual void notifyPorts() = 0;
    virtual void execute(Port* ch) = 0;
            void notify(const int, ...);
           
            void updateNextApp(Port*);
            void initializeLUU(BIP::Time& );
            bool inNextPortList(Port*);
    virtual void dump(FILE* file) const;

};

class CtrlState {
  /*** Private data members ***/
  private:
    char*   mName;
  /*** Constructoris & Dectructors ***/
  public:
    CtrlState(const char* = NULL);
    ~CtrlState();

    inline char* name() { return mName; }
};

#endif /* ATOM_H_*/
