/*******************************************************************************
 *                            C++ Verimag
 *    
 *
 *------------------------------------------------------------------------------
 * 	            Distributed and Complex Systems - Gires
 *------------------------------------------------------------------------------
 * executable.h
 *------------------------------------------------------------------------------
 *  Abstract objects for BIP2 Engine
 *  + class Executable  : executable object (port or interaction)
 *------------------------------------------------------------------------------
 * Created by Jacques COMBAZ on Mon Jan 04 2010
 *
 ******************************************************************************/

#ifndef EXECUTABLE_H
#define EXECUTABLE_H

#include "bip-time.h"
#include "clock.h"
#include "constraint.h"

#include <iostream>

/**********************************************
 * Class: Executable
 * Description: object executable by the Engine
 *   can be either an interaction or a port
 *********************************************/
class Executable {
  /*** Protected data members ***/
  private: 
    unsigned int mMark;                /* mark for filters */
    Intervals    mConstraint;          /* time constraint  */


  /*** Constructors & Dectructors ***/
  public:
    Executable();
    virtual ~Executable();

  /*** Public Data-Access Methods ***/
  public:
    inline        unsigned int     mark()                               const { return mMark; }
    inline        void             setMark(unsigned int m)                    { mMark = m; }
    inline        Intervals& constraint()                               { return mConstraint; }
    inline  const Intervals& constraint()                         const { return mConstraint; }

  /*** Public Utility Methods ***/
  public:
                  void          dominate(Executable&,                   const BIP::Time& = BIP::Time::ZERO);
                  void          dominate(Executable&, const Intervals&, const BIP::Time& = BIP::Time::ZERO);
    virtual       char*         label()                   = 0;
    virtual       void          execute(const BIP::Time&) = 0;
    virtual       void          actuate(const BIP::Time&) = 0;
                  BIP::Time     deadline(const BIP::Time& time) const { return constraint().deadline(time); }
                  BIP::Time     next(const BIP::Time& time)     const { return constraint().next(time); }

    friend std::ostream&  operator<<(std::ostream &, Executable&);
    friend std::ostream&  operator<<(std::ostream &, Executable*);
};

#endif

