

#ifndef PORT_H
#define PORT_H

#include <stdio.h>
#ifdef SYMBOLIC
#include  "cudd.h"
#endif

#include "structure.h"
#include "executable.h"
#include "connector.h"

class Executable;
class Atom;
class Connector;


/***************************
 * Class: Port
 * Description: port class
 **************************/
class Port: public Executable {
  /*** Private data members ***/
    char LABEL[1024];

  /*** Protected data members ***/
  protected:
    char*       mName;        /* Name of the port */
    Atom*       mAtomHolder;  /* holder for atomic port */
    Connector*  mConnHolder;  /* holer for conn port */
    bool        mSynced;      /* the holder syncs it     */
    bool        mInternal;    /* if internal port */
    Port*       mNext;        /* next legal (internal) port */
    Array<Connector,false> mLinks; /* all connectors connected to this port */
#ifdef SYMBOLIC
    DdNode*     mBdd;         /* BDD for port */
#endif

  /*** Constructors & Dectructors ***/
  public:
    Port(const char*, 
         const Atom*);
    Port(const char*,
         const Connector*);
    virtual ~Port();

  /*** Public Data-Access Methods ***/
  public:
    inline char*       name()                          const { return mName; }
    inline Atom*       atomHolder()                    const { return mAtomHolder; }
    inline Connector*  connHolder()                    const { return mConnHolder; } 
    inline bool        synced()                        const { return mSynced; }
    inline void        setSynced(bool synced)                { mSynced = synced; }
    inline bool        isInternal()                    const { return mInternal; }
    inline void        setInternal()                         { mInternal = true; }
    inline Port*       next()                          const { return mNext; }
    inline void        setNext(Port* port)                   { mNext = port; }
    inline Array<Connector,false>& links()                   { return mLinks; }
    inline void        resetLink() { for (int i = 0 ; i < mLinks.count() ; i++) { mLinks[i]->resetLink(); } }
#ifdef SYMBOLIC
    inline DdNode*     bdd()         const                   { return mBdd; }
    inline void        setBdd(DdNode* bdd)                   { mBdd= bdd; }
#endif

  /*** Public Utility Methods ***/
  public:
            void  notifyDependentPorts();
            void  execute(const BIP::Time&);
    virtual void  actuate(const BIP::Time&);
            void  execute();
            char* label();
            void setLink(Connector*);
#ifdef SYMBOLIC
            void initBdd();
#endif

  /*** Public Methods invoked from appln. ***/
  public:
    bool complete() const;
    bool guard();
};

#endif
