
#ifndef STRUCTURE_H
#define STRUCTURE_H

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

/*
 *
 */

class BitVector;

template <class C> class List;

template <class C,bool O> class Array;

template <class K,class C> class HashTable;

/*
 *
 */

class BitVector {

 protected:
  static const int SIZE = 4;
  unsigned m_data[SIZE];
  
 public:
  BitVector();
  BitVector(const BitVector&);
  BitVector(const int);
  ~BitVector();
  
  BitVector& operator=(const BitVector&);
  BitVector& operator&=(const BitVector&);
  BitVector& operator|=(const BitVector&);
  BitVector& operator<<(int);
  BitVector& operator>>(int);
  
  inline bool operator==(const BitVector& bv) const {
    bool eq = true;
    for(int i = 0; i < SIZE && eq; i++) {
      if (m_data[i] != bv.m_data[i]) {
        eq = false;
      }
    }
    return eq;
  }

  inline bool operator<=(const BitVector& bv) const {
    bool inc = true;
    for(int i = 0 ; i < SIZE && inc; i++) {
      if ((m_data[i] | bv.m_data[i]) != bv.m_data[i]) {
        inc = false;
        break;
      }
    }
    return inc;
  }

  inline bool operator<(const BitVector& bv) const {
    bool inc = true, eq = true;
    for(int i = 0; i < SIZE && inc; i++) {
      if ((m_data[i] | bv.m_data[i]) != bv.m_data[i]) {
       inc = false;
      }
      if (m_data[i] != bv.m_data[i]) eq = false;
    }
    return inc && !eq;
  }

  inline bool operator!=(const BitVector& bv) const
    { return ! (*this == bv); }
  inline bool operator>(const BitVector& bv) const
    { return bv < *this; }
  inline bool operator>=(const BitVector& bv) const
    { return bv <= *this; }
  
  inline int in(int k) const
    { return (m_data[k / 32] & ((unsigned)1 << (k % 32))); }
  inline void add(int k)
    { m_data[k / 32] |=  ((unsigned) 1 << (k % 32)); }
  inline void rem(int k)
    { m_data[k / 32] &= ~((unsigned) 1 << (k % 32)); }

  inline unsigned hash(unsigned base) const { unsigned int ret = 0; for (int i = 0 ; i < SIZE ; i++) ret += m_data[i]; return ret % base; }

  friend std::ostream& operator<<(std::ostream&, const BitVector&);

  static BitVector EMPTY;
  static BitVector SINGLE;
};

/*
 *
 */

template <class C> class List {

 public:
  struct Node { 
    Node *next,*prev;
    C* elem;
  };

 protected:
  Node *m_head, *m_tail;

 public:
  List() {
    m_head = m_tail = NULL;
  }
  ~List() {
    while (m_head != NULL) 
      remove(m_head);
  }
  
  inline Node* head() const 
    { return m_head; }
  inline Node* tail() const 
    { return m_tail; }
  inline Node* next(Node* node) const 
    { return node->next; }
  inline Node* prev(Node* node) const
    { return node->prev; }
  inline C* elem(Node* node) const
    { return node->elem;}
  
  void insert(const C* object) {
    Node* n = new Node;
    n->elem = (C*) object;
    n->prev = NULL;
    n->next = m_head;
    if (m_head == NULL) {
      m_head = m_tail = n;
    }
    else {
      m_head->prev = n;
      m_head = n;
    }
  }
  void append(const C* object) {
    Node* n = new Node;
    n->elem = (C*) object;
    n->prev = m_tail;
    n->next = NULL;
    if (m_tail == NULL) {
      m_head = m_tail = n;
    }
    else {
      m_tail->next = n;
      m_tail = n;
    }
  }
  void remove(Node* node) {
    if (node->prev == NULL)
      m_head = node->next;
    else
      node->prev->next = node->next;
    if (node->next == NULL) 
      m_tail = node->prev;
    else
      node->next->prev = node->prev;
    delete node;
  }
  C* take() {
    C* object = NULL;
    if (m_head != NULL) {
      object = m_head->elem;
      remove( m_head );
    }
    return object;
  }
  inline int isEmpty() const
    { return m_head == NULL; }
  int size() const {
    int s = 0;
    Node* tmp = m_head;
    while (tmp) {
      s++;
      tmp = tmp->next;
    }
    return s;
  }
};

/*
 *
 */

template <class C,bool Own> class Array {

 protected:
  C* *m_elem;
  int m_count;
  int m_size;
  
 protected:
  void resize() {
    int growth = (m_size / 3) + 1;
    m_size += growth;
    m_elem = (C**) realloc( m_elem, sizeof(C*) * m_size);
  }
  
 public:
  Array(const int size = 4) {
    m_size = size;
    m_elem = (C**) malloc( sizeof(C*) * m_size);
    m_count = 0;
  }
  Array(const int size, const C* object[]) {
    m_size = size;
    m_elem = (C**) malloc( sizeof(C*) * m_size);
    m_count = m_size;
    for(int i = 0; i < m_size; i++)
      m_elem[i] = (C*) object[i];
  }
  ~Array() {
    if (Own)
      for(int i = 0; i < m_count; i++)
	delete m_elem[i];
    free(m_elem);
  }
  
  inline int count() const 
    { return m_count; }
  inline C* operator[](const int k) const
    { return m_elem[k]; }
  void add(const C* object) {
    if (m_count == m_size) resize();
    m_elem[m_count++] = (C*) object;
  }
 //new code used by BDD
 inline bool Belongs(const C* a)  {
    for(int i=0;i<m_count;i++)
      if(m_elem[i]==a)
        return 1;
    return 0;

  }

};

/*
 *
 */

template <class K, class C> class HashTable {

protected:
  static const int SIZE = 511;
  struct Node {
    Node* next;
    C* elem;
  } * m_entry[SIZE];
  int m_count;

  void purge() {
    for(int i = 0; i < SIZE; i++)
      while (m_entry[i] != NULL) {
	Node* n = m_entry[i];
	m_entry[i] = n->next;
	delete n->elem;
	delete n;
      }
    m_count = 0;
  }

public:
  HashTable() {
    for(int i = 0; i < SIZE; i++)
      m_entry[i] = NULL;
    m_count = 0;
  }

  ~HashTable() {
    purge();
  }

  inline int count() const
    { return m_count; }

  C* store(const K& k) {
    int hash = k.hash(SIZE);
    Node* n = NULL;
    for(n = m_entry[hash]; n != NULL; n=n->next) 
      if (k == n->elem->key())
	break;
    if (n == NULL) {
      n = new Node();
      n->elem = new C(k);
      n->next = m_entry[hash];
      m_entry[hash] = n;
      m_count++;
    }
    return n->elem;
  }

};

/*
 *  
 *  heap   : implements dynamic heaps
 *  author : Marius Bozga
 *  note   : taken from IF src 
 *
 */

// warnings
// - object indexing must start at 1, not at 0
// - count denotes here the last used position

template <class C>
class BipHeap {

 protected:
  int m_length;
  C* *m_objects;
  int m_count;

 public:
  BipHeap() {
    m_length = 4;
    m_objects = (C**) malloc(sizeof(C*) * m_length);
    m_count = 0;
  }
 
  virtual ~BipHeap() {
    free(m_objects);
  }

 public:
  inline int getCount() const
    { return m_count; }

 public:
  C* get() {
    if (m_count == 0)
      return NULL;

    C* top = m_objects[1];
    m_objects[1] = m_objects[m_count--];

    // balance
    for(int j = 1; j <= m_count; ) {
      int i = j * 2;
      if (i > m_count)
        break;
      if (i + 1 <= m_count &&
          m_objects[i+1]->getPriority() > m_objects[i]->getPriority())
        i = i + 1;
      if (m_objects[j]->getPriority() >= m_objects[i]->getPriority())
        break;

      C* tmp = m_objects[j];
      m_objects[j] = m_objects[i];
      m_objects[i] = tmp;

      j = i;
    }

    return top;
  }

  void put(const C* object) {
    if (m_count == m_length - 1) {
      m_length += m_length;
      m_objects = (C**) realloc(m_objects, sizeof(C*) * m_length);
    }

    m_objects[++m_count] = (C*) object;
    
    // balance
    for(int i = m_count; i > 1;) {
      int j = i / 2;

      if (m_objects[j]->getPriority() >= m_objects[i]->getPriority())
        break;

      C* tmp = m_objects[j];
      m_objects[j] = m_objects[i];
      m_objects[i] = tmp;

      i = j;
    }
  }

};


#endif

