#ifndef PORT_H
#define PORT_H

#include "constraint.h"
class Atom;
class Connector;

/***************************
 * Class: Port
 * Description: port class
 **************************/
class Port {
	
  public:
	typedef std::vector<Connector*> connectors_type;
	
	connectors_type mLinks; /* all connectors connected to this port */
	
	
	 /*** Protected data members ***/
  protected:
    char*      mName;       /* Name of the port */
    Atom*      mAtomHolder; /* holder for atomic port */
    Connector* mConnHolder; /* holer for conn port */
    bool       mSynced;     /* the holder syncs it     */
  
    bool       mElected;    /* the scheduler elects it */
    bool       mConnected;  /* i.e. connected, else internal label */
    bool       mInternal;   /* if internal port */
    
    Intervals  mConstraint;  /* time constraint  */
    
    Urgency    mPortUrgency;
    bool       mGuardApp;
  
    BIP::Time  mLowerApp  ;   /*approximated lower bound of time constraint*/
    BIP::Time  mUpperApp ;   /*approximated Upper bound of time constraint*/
    Urgency    mUrgencyApp;   /*approximated Urgency of time constraint*/
 
    
    bool        mInput;       /* the port is an input     */
    bool        mOutput;      /* the port is an output    */
    bool        mSense;       /* the port is an input     */
    bool        mActuate;      /* the port is an output    */
  /*** Constructors & Dectructors ***/
  public:
    Port(const char*, 
         Atom*);
    Port(const char*,
         Connector*);
    virtual ~Port();

  /*** Public Data-Access Methods ***/
  public:
    inline char*      name()        const      { return mName; }
    inline Atom*      atomHolder()  const      { return mAtomHolder; }
    inline Connector* connHolder()  const      { return mConnHolder; } 
    inline bool       synced()      const      { return mSynced; }


    inline void       setSynced(bool synced)   { mSynced = synced; }
   
    inline bool       elected()     const      { return mElected; }
    inline void       setElected(bool elected) { mElected = elected; }
    inline bool       isConnected() const      { return mConnected; }
    inline void       setConnected()           { mConnected = true; }
    inline bool       isInternal()  const      { return mInternal; }
    inline void       setInternal()            { mInternal = true; }
    inline        Intervals& constraint()                               { return mConstraint; }
    inline  const Intervals& constraint()                         const { return mConstraint; }
    inline        void       setConstraint(Intervals&  constraint)      {mConstraint = constraint;}
    inline        bool       guardApp()                                 { return mGuardApp; }
    inline  const bool       guardApp()                           const { return mGuardApp; }
    inline        void    setGuardApp(bool b)                           { mGuardApp = b;}
    inline        BIP::Time& lowerApp()                                 { return mLowerApp; }
    inline        BIP::Time& upperApp()                                 { return mUpperApp; }
    inline        Urgency    urgencyApp()                               { return mUrgencyApp; }

        inline bool        isInput()                       const { return mInput; }
        inline void        setInput()                            { mInput = true; }  
        inline bool        isOutput()                      const { return mOutput; }
        inline void        setOutput()                            { mOutput = true; }
        inline bool        isSense()                       const { return mSense; }
        inline void        setSense(bool sense)                  { mSense = sense; }
        inline bool        isActuate()                     const { return mActuate; }
        inline void        setActuate(bool actuate)              { mActuate = actuate; }
  
    connectors_type& links()
      {
        return mLinks;
      }
  /*** Public Utility Methods ***/
  public:
    void execute();
    void addLink(Connector*);
    bool holderReady();
    void holderGoBusy(int&);
    void triggerAtom();
    void updateApp();
    void initializeLUU(BIP::Time& );
    void setLink(Connector*);
    bool isNext();
    void initializeNextPorts(BIP::Time& );
    void resetLink() ;
  /*** Public Methods invoked from appln. ***/
  public:
    bool complete() const;
    bool guard();

};

#endif
