# - Try to find BIP
# Once done this will define
#
#  BIP2_FOUND - system has BIP
#  BIP2_INCLUDE_DIR - the BIP include directory
#  BIP2_LIBRARIES - the libraries needed to use BIP
#  BIP2_LIBRARY_DIR - the BIP library directory
#  add_bip2_system(targetName, fileName) - a macro to compile a BIP system in file fileName
#  add_bip2_include_directory(targetName, includeDir) - a macro to add a directory to the search list of the bip compiler for the target targetName
#  add_bip2_package(targetName, fileName) - a macro to compile a BIP package in file fileName

# Copyright 2008, 2009, 2010, VERIMAG, <bip-support.VERIMAG@imag.fr>
#     Matthieu Gallien, <matthieu.gallien@imag.fr>
#
# Redistribution and use is allowed according to the terms of the BSD license.
# For details see the accompanying COPYING-CMAKE-SCRIPTS file.


INCLUDE(CheckCXXCompilerFlag)

if (BIP2_INCLUDE_DIR AND BIP2_LIBRARIES)
   # in cache already
   set(BIP2_FIND_QUIETLY TRUE)
endif (BIP2_INCLUDE_DIR AND BIP2_LIBRARIES)

set(BIP2_HOME_BIN "$ENV{BIP2_HOME}/bin")

find_path(
  BIPC_PATH
    NAMES bipc
    PATHS ENV PATH ${BIP2_HOME_BIN}
    DOC "Path to bipc"
)

if (BIPC_PATH STREQUAL "BIPC_PATH-NOTFOUND")
else (BIPC_PATH STREQUAL "BIPC_PATH-NOTFOUND")
  message("bipc found in ${BIPC_PATH}")

  CHECK_CXX_COMPILER_FLAG("-std=c++0x" CXX_HAS_INCOMING_STANDARD_CPLUSPLUS_OPTION)
  if (${CXX_HAS_INCOMING_STANDARD_CPLUSPLUS_OPTION})
    set(CMAKE_CXX_FLAGS_DEBUG "${CMAKE_CXX_FLAGS_DEBUG} -std=c++0x")
    set(CMAKE_CXX_FLAGS_RELWITHDEBINFO "${CMAKE_CXX_FLAGS_RELWITHDEBINFO} -std=c++0x")
    set(CMAKE_CXX_FLAGS_RELEASE "${CMAKE_CXX_FLAGS_RELEASE} -std=c++0x")
    set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++0x")
  endif (${CXX_HAS_INCOMING_STANDARD_CPLUSPLUS_OPTION})

  find_program(
    BIPC_EXECUTABLE
      NAMES bipc
      PATHS ${BIPC_PATH}
      DOC "bipc"
  )

  message("bipc: ${BIPC_EXECUTABLE}")

  if (NOT DEFINED BIPMODE)
    set(BIPMODE "mono")
  endif (NOT DEFINED BIPMODE)


  if (${BIPMODE} STREQUAL "mono")
    set(BIPC_FLAGS "--genC-execute")
  else (${BIPMODE} STREQUAL "mono")
    set(BIPC_FLAGS "--genC-multi")
  endif (${BIPMODE} STREQUAL "mono")

  set(BIP2_LIBRARIES compo)

  string(REPLACE "/bin" "" BIP2_ROOT_PATH ${BIPC_PATH})
  set(BIP2_INCLUDE_DIR ${BIP2_ROOT_PATH}/include/${BIPMODE})
  set(BIP2_LIBRARY_DIR ${BIP2_ROOT_PATH}/lib/${BIPMODE})

  function(set_bip2_include_directory targetName includeDir)
    set(${targetName}_BIP_INCLUDES "-I ${includeDir}" CACHE INTERNAL "include directories for target ${targetName}" FORCE)
  endfunction(set_bip2_include_directory)

  function(add_bip2_include_directory targetName includeDir)
    set(${targetName}_BIP_INCLUDES ${${targetName}_BIP_INCLUDES} "-I ${includeDir}" CACHE INTERNAL "include directories for target ${targetName}")
  endfunction(add_bip2_include_directory)

  function(add_bip2_model fileName)
    get_filename_component(targetName ${fileName} NAME_WE)
    get_filename_component(${targetName}_FILENAME ${fileName} NAME)
    get_filename_component(${targetName}_PATH ${fileName} PATH)
    set(${targetName}_DEPENDENCIES_FILE "${${targetName}_FILENAME}.dependency.cmake")
    string(REPLACE ".bip" ".model" ${targetName}_GENERATED_MODEL_FILE ${${targetName}_FILENAME})
    set(${targetName}_GENERATED_MODEL_FILE ${${targetName}_GENERATED_MODEL_FILE} CACHE INTERNAL "generated model file name for target ${targetName}" FORCE)
    string(REPLACE ".bip" ".timod" ${targetName}_GENERATED_TIME_MODEL_FILE ${${targetName}_FILENAME})
    set(${targetName}_GENERATED_SOURCE_FILE ${${targetName}_FILENAME}.C CACHE INTERNAL "generated source file name for target ${targetName}" FORCE)
    set(${targetName}_GENERATED_HEADER_FILE ${${targetName}_FILENAME}.h)
    set(${targetName}_GENERATED_MAKEFILE ${${targetName}_FILENAME}.mk)

    set(${targetName}_STATIC_DEPENDENCIES ${fileName})
    foreach(staticDependency ${ARGN})
        set(${targetName}_STATIC_DEPENDENCIES ${${staticDependency}_GENERATED_MODEL_FILE} ${${targetName}_STATIC_DEPENDENCIES})
    endforeach(staticDependency ${ARGN})

    include(${${targetName}_DEPENDENCIES_FILE} OPTIONAL)

    add_custom_command(
        OUTPUT ${${targetName}_GENERATED_SOURCE_FILE} ${${targetName}_GENERATED_HEADER_FILE} ${${targetName}_GENERATED_MODEL_FILE} ${${targetName}_GENERATED_TIME_MODEL_FILE} ${${targetName}_GENERATED_MAKEFILE}
        COMMAND ${BIPC_EXECUTABLE}
        ARGS -f ${fileName} ${BIPC_FLAGS} ${${targetName}_BIP_INCLUDES}
        DEPENDS ${${targetName}_STATIC_DEPENDENCIES} ${${targetName}_PACKAGE_DEPENDENCIES}
        COMMENT "bipc ${${targetName}_FILENAME}"
    )
  endfunction(add_bip2_model)
endif (BIPC_PATH STREQUAL "BIPC_PATH-NOTFOUND")

include(FindPackageHandleStandardArgs)

# handle the QUIETLY and REQUIRED arguments and set BIP2_FOUND to TRUE if
# all listed variables are TRUE
find_package_handle_standard_args(BIP DEFAULT_MSG BIP2_LIBRARIES BIP2_INCLUDE_DIR)

mark_as_advanced(BIP2_INCLUDE_DIR BIP2_LIBRARIES)
