#ifndef DOLC_H
#define DOLC_H

/************************************************************************
 * do not add code to this header
 ************************************************************************/

/**
 *  Define the DOL process handler scheme.
 *  - Local variables are defined in structure LocalState. Local
 *    variables may vary from different processes.
 *  - The ProcessInit function pointer points to a function which
 *    initializes a process.
 *  - The ProcessFire function pointer points to a function which
 *    performs the actual computation. The communication between
 *    processes is inside the ProcessFire function.
 *  - The WPTR is a placeholder for callback. One can just
 *    leave it blank.
 */

//structure for local memory of process
typedef struct _dolc_local_states *DOLCLocalState;

//additional behavioral functions could be declared here
typedef void (*DOLCProcessInit)(struct _process*);
typedef void (*DOLCProcessFire)(struct _process*);
typedef void *WPTR;

//process handler
struct _process;

typedef struct _process {
    DOLCLocalState     local;
    DOLCProcessInit    init;
    DOLCProcessFire    fire;
    WPTR           wptr; //placeholder for wrapper instance
} DOLCProcess;


//macros to deal with iterated ports

/**
 * macro to create a variable to store a port name
 *
 * @param name name of the variable
 */
 
#define MAX_PORT_NAME_LENGTH 255
#define CREATEPORTVAR(name) char name[MAX_PORT_NAME_LENGTH]

/**
 * Create the port name of an iterated port based on its basename and the
 * given indices.
 *
 * @param port buffer where the result is stored (created using
 *             CREATEPORTVAR)
 * @param base basename of the port
 * @param number_of_indices number of dimensions of the port
 * @param index_range_pairs index and range values for each dimension
 */
 
#define CREATEPORT(port, base, number_of_indices, indices...) \
  createPort(port, base, number_of_indices, indices)
  
char* createPort(char*, const char*, int, int, ...);
void DOLC_detach(DOLCProcess*);


typedef struct {
    int valid; // validity bit
    void* data; // pointer to actual data
	void* ptr;  // alias to data to be compatible with ETHZ
    unsigned int size; //size data element
} DOLCData;

#define DOLCDataVar(type, var) type dolcdata_##var; \
DOLCData var; var.valid=false; var.data = &dolcdata_##var; var.ptr = var.data; var.size = sizeof(type)
#define PHASE(name)

// put the used function prototype declaration here so that 
// codegen parser does not warn about implicit function declaration
int DOLC_write(void* port, DOLCData* addr, unsigned len, DOLCProcess* p);
void DOLC_read(void* port, DOLCData* addr, unsigned len, DOLCProcess* p);

#endif

